import 'dart:convert';
import 'dart:developer';
import 'dart:collection';
import 'dart:developer';

import 'package:easynetflutter/easynet/log/Logger.dart';
import 'package:shared_preferences/shared_preferences.dart';

class SessionDataBase {
  static String _SESSION_LIST_NAMES="_SESSION_LIST_NAMES";
  static Logger logger = Logger(className: "SessionDataBase");
  static Map<String, String> mapSessionName = Map();

  static clearSession() async {
    logger.w("ClearSession: Removendo ${mapSessionName.length} objetos.");
    if (mapSessionName !=  null && mapSessionName.length > 0) {
      //mapSessionName.forEach((k,v) => removeObject(k));
      mapSessionName.forEach((k,v) => logger.w("Excluindo ${k}"));
      mapSessionName.clear();
    }
  }

  static Future<Map<String, dynamic>> getMapSession() async{
    String json = await getObject(_SESSION_LIST_NAMES);
    logger.w("Lista com o nomes da sessao ${json}");

    return json!= null && json.trim().length > 0? jsonDecode(json):{};
  }

  static void loadMapSession() async{
    mapSessionName = await getMapSession();
  }

  static void saveMapSession(String sessionName) async {
    await loadMapSession();
    if (sessionName != null && !mapSessionName.containsKey(sessionName)) {
      logger.w("Salvando o nome da Sessao ${sessionName}");
      Map<String, String> newEntry = {sessionName: sessionName};
      mapSessionName.addAll(newEntry);
      saveOject(mapSessionName, _SESSION_LIST_NAMES);
    }
  }

  static void removeMapSession(String sessionName) async {
    await loadMapSession();
    mapSessionName.remove(sessionName);
  }

  static Future<String> getObject(String sessionName) async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    String data = prefs.getString(sessionName);
    if (data != null) {
      logger.w("obtendo objeto $sessionName : $data ");
      return data;
    } else {
      return null;
    }
  }

  static removeObjectMap(String sessionName) async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    prefs.remove(sessionName);
    logger.w("removendo objeto interno $sessionName");
  }

  static removeObject(String sessionName) async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    prefs.remove(sessionName);
    logger.w("removendo objeto $sessionName");
    removeMapSession(sessionName);
  }

  static Future saveOject(Object value, String sessionName) async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    String jsonStr = jsonEncode(value);
    prefs.setString(sessionName, jsonStr);
    saveMapSession(sessionName);
    logger.w("salvando objeto $value " + value.toString());
  }

  static Future saveString(String sessionName, String value) async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    prefs.setString(sessionName, value);
    saveMapSession(sessionName);
  }

  static Future saveInt(String sessionName, int value) async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    prefs.setInt(sessionName, value);
    saveMapSession(sessionName);
  }

  static Future saveBool(String sessionName, bool value) async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    prefs.setBool(sessionName, value);
    saveMapSession(sessionName);
  }

  static Future saveDouble(String sessionName, double value) async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    prefs.setDouble(sessionName, value);
    saveMapSession(sessionName);
  }

  static Future saveListString(String sessionName, List<String> value) async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    prefs.setStringList(sessionName, value);
    saveMapSession(sessionName);
  }
}